#!/usr/bin/python3.5

#
#  engine
#
#  Playout Daemon for autoradio project
#
#
#  Copyright (C) 2017-2018 Gottfried Gaisbauer <gottfried.gaisbauer@servus.at>
#
#  This file is part of engine.
#
#  engine is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  any later version.
#
#  engine is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with engine. If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import signal
import unittest

from modules.scheduling.scheduler import AuraScheduler
from modules.communication.liquidsoap.communicator import LiquidSoapCommunicator
from modules.communication.redis.adapter import ServerRedisAdapter
from modules.web.routes import Routes
from modules.monitoring.diskspace_watcher import DiskSpaceWatcher
from libraries.base.logger import AuraLogger

from testing.test import TestConfig, TestLogger

class Aura(AuraLogger):
    server = None
    messenger = None
    controller = None

    # ------------------------------------------------------------------------------------------ #
    def __init__(self):
        super(Aura, self).__init__()

    def test_yourself(self):
        self.run_test(TestConfig)
        self.run_test(TestLogger)

    def run_test(self, testingClass):
        runner = unittest.TextTestRunner()
        result = runner.run(unittest.makeSuite(testingClass))

    def startup(self):

        server = object

        #        self.controller = AuraController(self.config)
        # create scheduler and ls_communicator
        self.liquidsoapcommunicator = LiquidSoapCommunicator(self.config)
        self.scheduler = AuraScheduler(self.config)

        # give both a reference of each other
        self.liquidsoapcommunicator.scheduler = self.scheduler
        self.scheduler.liquidsoapcommunicator = self.liquidsoapcommunicator

        # create the redis adapter
        self.messenger = ServerRedisAdapter(self.config)
        self.messenger.scheduler = self.scheduler
        self.messenger.liquidsoapcommunicator = self.liquidsoapcommunicator

        self.diskspace_watcher = DiskSpaceWatcher(self.config, self.logger, self.liquidsoapcommunicator)
        self.diskspace_watcher.start()

        

        def receive_signal(signum, stack):
            print("received signal")
            server.reload()

        signal.signal(signal.SIGUSR1, receive_signal)

        # and finally wait for redis message
        self.join_comm()

        # start the web service
        self.start_web_service()



    def join_comm(self):
        # start listener thread
        self.messenger.start()

    def start_web_service(self):
        try:
            self.logger.info("Listening on Port 5000 for API or Webcalls")
            Routes(self.scheduler, self.liquidsoapcommunicator, self.messenger, self.config)
        except OSError as e:
            self.messenger.halt()
            self.logger.critical("AuraEngine already running? Exception: " + e.strerror + ". Exiting...")
            os._exit(0)


# # ## ## ## ## ## # #
# # ENTRY FUNCTION # #
# # ## ## ## ## ## # #
def main():
    aura = Aura()

    if len(sys.argv) >= 2 and sys.argv[1] == "--na-gspiast-di-?":
        aura.test_yourself()
    else:
        aura.startup()


# # ## ## ## ## ## ## # #
# # End ENTRY FUNCTION # #
# # ## ## ## ## ## ## # #


if __name__ == "__main__":
    main()
